#lang slideshow

(require (lib "code.ss" "slideshow")
         "utils/utils.ss"
         "utils/colors.ss"
         "utils/alg.ss")

(define (at* s)
  (colorize (t s) (current-id-color)))
(define (at s)
  (inset (at* s) (* 4 (current-line-sep))))

;; ----------------------------------------
(slide (titlet "Part 1"))

(define reduction
  (code (+ (* 4 3) (- 8 7)) #,sym:implies (+ 12 (- 8 7)) #,sym:implies (+ 12 1)))

(slide
 #:title "Racket vs. Algebra"
 (blank)
 'alts
 (list
  (list
   (para " ")
   reduction)
  (list
   (para #:width client-w
         "In Racket, we have a specific order for evaluating sub-expressions:")
   reduction))
 'next
 (blank)
 (blank)
 (para #:width client-w
       "In Algebra, order doesn't matter:")
 (para #:fill? #f (at "(4\u00B73)+(8-7)") sym:implies (at "12+(8-7)") sym:implies (at "12+1"))
 (bt "or")
 (para #:fill? #f (at "(4\u00B73)+(8-7)") sym:implies (at "(4\u00B73)+1") sym:implies (at "12+1")))

(slide
 #:title "Algebraic Shortcuts"
 (para "In Algebra, if we see")
 (vl-append (current-line-sep)
            (at "f(x, y) = x")
            (at "g(z) = ...")
            (at "f(17, g(g(g(g(g(18))))))"))
 (para "then we can go straight to")
 (at "17")
 (para "because the result of all the" (at* "g") "calls"
       "will not be used")
 'next
 (blank)
 (blank)
 (colorize (para #:fill? #f "But why would a programmer write something like that?")
           BlueColor))

(slide
 #:title "Avoiding Unnecessary Work"
 (scode (code:contract layout-text : string w h -> pict)
        (define (layout-text txt w h)
          (local [(define lines
                    (code:comment "lots of work to flow a paragraph")
                    ...)]
            
            (make-pict w
                       h
                       (lambda (dc x y)
                         (code:comment "draw paragraph lines")
                         ...))))
        ...
        (define speech (layout-text "Four score..."
                                    800
                                    600))
        ...
        (pict-width speech)))

(slide
 #:title "Avoiding Unnecessary Work"
 (scode (code:contract read-all-chars : file -> list-of-char)
        (define (read-all-chars f)
          (if (at-eof? f)
              empty
              (cons (read-char f) (read-all-chars f))))
        ...
        (define content (read-all-chars (open-file user-file)))
        (if (equal? (first content) #\#)
            (process-file (rest content))
            (error 'parser "not a valid file"))))

(slide
 #:title "Recursive Definitions"
 (scode (code:contract numbers-from : int -> list-of-int)
        (define (numbers-from n)
          (cons n (numbers-from (add1 n))))
        ...
        (define nonneg (numbers-from 0))
        (list-ref nonneg 10675)))

(slide
 #:title "Lazy Evaluation"
 (para "Languages like Racket, Java, and C are called" (dt "eager"))
 (item "An expression is evaluated when it is encountered")
 'next
 (blank)
 (blank)
 (para "Languages that avoid unnecessary work are called" (dt "lazy"))
 (item "An expression is evaluated only if its result is needed"))

;; ----------------------------------------
(slide (titlet "Part 2"))

(slide
 #:title "Lazy Evaluation in DrRacket"
 (vc-append
  (current-line-sep)
  (para "The " (tt "plai-lazy") "package adds support for")
  (tt "#lang plai-lazy"))
 'next
 (blank)
 (blank)
 (para #:width (* 0.9 client-w) "For coverage reports:")
 (para  "In the" (bt "Choose Language...") "dialog, click" (bt "Show Details")
        "and then" (bt "Syntactic test suite coverage"))
 (para #:align 'right
       (scale/improve-new-text
        (t "(Works for both eager and lazy languages)")
        0.80))
 (blank)
 (vl-append
  (* 2 (current-line-sep))
  (item #:fill? #f (colorize (t "Black") "black") "means evaluated at least once")
  (item #:fill? #f (colorize (t "Orange") "orange") "means not yet evaluated")
  (item #:fill? #f "Normal coloring is the same as all black")))

;; ----------------------------------------
(slide (titlet "Part 3"))

(define letrec-title
  (hbl-append (code letrec) (titlet " Interepreter in ") (tt "plai-lazy")))

(slide
 #:title letrec-title
 #:layout 'top
 (para #:width client-w 
       "Doesn't work because result of"
       (code set-box!) "is never used:")
 (blank)
 (sscode
  (define (interp a env)
    (type-case ExprC a
      ...
      [letrecC (n rhs body)
               (let ([b (box (numV 42))])
                 (let ([new-env (extend-env
                                 (bind n b)
                                 env)])
                   (begin
                     #,(hilite (code (set-box! b (interp rhs new-env))))
                     (interp body new-env))))]))))

(slide
 #:title letrec-title
 #:layout 'top
 (para #:width client-w "Working implementation is more direct:")
 (blank)
 (sscode
  (define (interp a env)
    (type-case ExprC a
      ...
      [letrecC (n rhs body)
               (letrec ([new-env
                         (extend-env
                          (bind n (interp rhs new-env))
                          env)])
                 (interp body new-env))]))))

;; ----------------------------------------
(slide (titlet "Part 4"))

(slide
 #:title "Lazy Language"
 (grammar-table
  (list (nonterm "Expr") eqls (nonterm "Num") (blank)
        (blank)          -or- (code #,(nonterm "Sym")) (blank)
        (blank)          -or- (code {+ #,(nonterm "Expr") #,(nonterm "Expr")}) (blank)
        (blank)          -or- (code {* #,(nonterm "Expr") #,(nonterm "Expr")}) (blank)
        (blank)          -or- (code {lambda {#,(nonterm "Sym")} #,(nonterm "Expr")}) (blank)
        (blank)          -or- (code {#,(nonterm "Expr") #,(nonterm "Expr")}) (blank)))
 'next
 (blank)
 (vl-append
  (current-line-sep)
  (code {{lambda {x} 0} {+ 1 {lambda {y} 2}}} #,sym:implies 0)
  (code {{lambda {x} x} {+ 1 {lambda {y} 2}}} #,sym:implies #,(it "error")))
 'next
 (blank)
 (code {let {[x {+ 1 {lambda {y} 2}}]}
         0} #,sym:implies 0))

;; ----------------------------------------
(slide (titlet "Part 5"))

(define (app-code fun-expr arg-interp)
  (scode
   (define (interp a env)
     (type-case ExprC a
       ...
       [appC (fun arg)
             (type-case Value #,fun-expr
               [closV (n body c-env)
                      (interp body
                              (extend-env
                               (bind n (#,arg-interp arg env))
                               c-env))]
               [else (error 'interp "not a function")])]))))

(define fun-expr
  (code (interp fun env)))

(define-syntax red-id
  (syntax-rules ()
    [(_ id)
     (hilite (code id))]))

(define-syntax white-id
  (syntax-rules ()
    [(_ id)
     (hilite #:color "white" (code id))]))

(slide
 #:title "Implementing Laziness"
 'alts
 (list
  (list
   (item #:bullet (colorize (t "Option #1:") BlueColor)
         "Run the interpreter in " (tt "plai-lazy") "!")
   'next
   (blank)
   (app-code fun-expr (red-id interp))
   (blank)
   (para #:fill? #f
         (code n) "never used" sym:implies
         (code interp) "call never evaluated"))
  (list
   (item #:bullet (colorize (t "Option #2:") BlueColor)
         "Use " (tt "plai-typed") "and explicitly delay" 
         (code arg) "interpretation")
   'next
   (blank)
   (app-code fun-expr (red-id suspendV))
   (blank)
   (para "where" (code suspendV) "is a new kind of" (code Value)))))

(slide
 #:title "Values"
 (code (define-type Value
         [numV (n : number)]
         [closV (arg : symbol)
                (body : ExprC)
                (env : Env)]
         [suspendV (body : ExprC)
                   (env : Env)])))

;; ----------------------------------------
(slide (titlet "Part 6-7"))

(slide
 #:title "Forcing Evaluation for Number Operations"
 (code (interp #,(parsed {{lambda {x} {+ 1 x}} 10}) mt-env))
 'next
 (para #:fill? #f sym:implies (it "error: expected numV, got suspendV"))
 'next
 (blank)
 'alts
 (list
  (list
   (code #,sym:implies 
         (interp #,(parsed {+ 1 x})
                 (extend-env (bind 'x
                                   (suspendV #,(parsed 10) mt-env))
                             mt-env))))
  (list
   (scode
    (define (interp [a : ExprC] [env : Env]) : Value
      (type-case ExprC a
        ...
        [plusC (l r) (num+ #,(red-id (strict #,(white-id (interp l env))))
                           #,(red-id (strict #,(white-id (interp r env)))))]
        ...))
    code:blank
    (define (strict [v : Value]) : Value
      (type-case Value v
        [suspendV (b e) (strict (interp b e))]
        [else v]))))))

;; ----------------------------------------
(slide (titlet "Part 8"))

(slide
 #:title "Forcing Evaluation for Application"
 (code (interp #,(parsed {{lambda {f} {f 1}} {lambda {x} {+ x 1}}})
               mt-env))
 'next
 (blank)
 'alts
 (list
  (list
   (code #,sym:implies 
         (interp #,(parsed {f 1})
                 (extend-env
                  (bind 'f
                        (suspendV #,(parsed {lambda {x} {+ x 1}})
                                  mt-env))
                  mt-env))))
  (list
   (app-code (red-id (strict #,(white-id #,fun-expr))) (red-id suspendV)))))

;; ----------------------------------------
(slide (titlet "Part 9"))

(slide
 #:title "Redundant Evaluation"
 (code
  {{lambda {x} {+ {+ x x} {+ x x}}}
   {- {+ 4 5} {+ 8 9}}})
 (blank)
 (para "How many times is" (code {+ 8 9}) "evaluated?")
 'next
 (blank)
 (para "Since the result is always the same, we'd like to"
       "evaluate" (code {- {+ 4 5} {+ 8 9}}) "at most once"))

(define caching-title "Caching Strict Results")

(slide
 #:title caching-title
 (code (define-type Value
         ...
         [suspendV (body : ExprC)
                   (env : Env)
                   (done : (boxof (optionof Value)))])))

(slide
 #:title caching-title
 (scode
  (define (strict [v : Value]) : Value
    (type-case Value v
      [suspendV (b e d)
                (type-case (optionof Value) (unbox d)
                  [none ()
                        (let ([v (strict (interp b e))])
                          (begin
                            (set-box! d (some v))
                            v))]
                  [some (v) v])]
      [else v]))))

(slide
 #:title "Fix Up Interpreter"
 (code
  (define (interp a env)
    ....
    [appC (fun arg) 
          ... (suspendV arg env #,(red-id (box (none)))) ...])))

;; ----------------------------------------
(slide (titlet "Part 10"))

(define term-title "Terminology")

(slide
 #:title term-title
 'next
 (para (dt "Call-by-value") "means eager")
 (para #:align 'right "Racket, Java, C, Python...")
 'next
 (blank)
 (blank)
 (para (dt "Call-by-name") "means lazy, no caching of results")
 (para #:align 'right "... which is impractical")
 'next
 (blank)
 (blank)
 (para (dt "Call-by-need") "means lazy, with caching of results")
 (para #:align 'right "Haskell, Clean..."))

(slide
 #:title term-title
 (para (dt "Normal order") "vs" (dt "Applicative order"))
 (blank)
 (para #:align 'right "... good terms to avoid"))
